@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -42,6 +45,8 @@
 package org.netbeans.nbbuild;
 
 import java.io.BufferedReader;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -51,10 +56,12 @@ import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.URI;
+import java.net.URL;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Date;
+import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
@@ -63,20 +70,31 @@ import java.util.Properties;
 import java.util.StringTokenizer;
 import java.util.jar.Attributes;
 import java.util.jar.Attributes.Name;
+import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.zip.CRC32;
 import java.util.zip.ZipEntry;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
+import org.apache.tools.ant.AntClassLoader;
 import org.apache.tools.ant.BuildException;
+import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
+import org.apache.tools.ant.taskdefs.Copy;
 import org.apache.tools.ant.taskdefs.Jar;
 import org.apache.tools.ant.taskdefs.SignJar;
+import org.apache.tools.ant.types.FileSet;
+import org.apache.tools.ant.types.Path;
 import org.apache.tools.ant.types.ZipFileSet;
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
+import org.xml.sax.EntityResolver;
+import org.xml.sax.ErrorHandler;
+import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
+import org.xml.sax.SAXParseException;
 
 /** Makes a <code>.nbm</code> (<b>N</b>et<b>B</b>eans <b>M</b>odule) file.
  *
@@ -160,11 +178,7 @@ public class MakeNBM extends Task {
 		    first = false;
 		} else {
 		    int i;
-		    for (i = 0;
-			 i < line.length () &&
-			     Character.isWhitespace (line.charAt (i));
-			 i++)
-			;
+                    for (i = 0; i < line.length() && Character.isWhitespace(line.charAt(i)); i++) {}
 		    if (i < min) min = i;
 		}
 	    }
@@ -304,7 +318,7 @@ public class MakeNBM extends Task {
     private String targetcluster = null;
     private String jarSignerMaxMemory = "96m";
     private Blurb license = null;
-    private Blurb description = null;
+    private Blurb desc = null;
     private Blurb notification = null;
     private Signature signature = null;
     private long mostRecentInput = 0L;
@@ -313,7 +327,11 @@ public class MakeNBM extends Task {
     private ArrayList<String> locales = null;
     private ArrayList<Attributes> moduleAttributes = null;
     private Attributes englishAttr = null;
-    
+    private Path updaterJar;
+    private FileSet executablesSet;
+    private boolean usePack200;
+    private String pack200excludes;
+
     /** Try to find and create localized info.xml files */
     public void setLocales(String s) {
         locales = new ArrayList<String>();
@@ -335,6 +353,19 @@ public class MakeNBM extends Task {
     public void setFile(File file) {
         this.file = file;
     }
+
+    public void setUsePack200(boolean usePack200) {
+        this.usePack200 = usePack200;
+    }
+
+    public void setPack200Excludes(String pack200excludes) {
+        this.pack200excludes = pack200excludes;
+    }
+
+    /** List of executable files in NBM concatinated by ${line.separator}. */
+    public FileSet createExecutables() {
+        return (executablesSet = new FileSet());
+    }
     
     /** Module manifest needed for versioning.
      * @deprecated Use {@link #setModule} instead.
@@ -409,7 +440,7 @@ public class MakeNBM extends Task {
     }    
     public Blurb createDescription () {
         log(getLocation() + "The <description> subelement in <makenbm> is deprecated except for emergency patches, please ensure your module has an OpenIDE-Module-Long-Description instead", Project.MSG_WARN);
-	return (description = new Blurb ());
+        return desc = new Blurb();
     }
     public Signature createSignature () {
 	return (signature = new Signature ());
@@ -428,13 +459,18 @@ public class MakeNBM extends Task {
     public ZipFileSet createMain () {
         return (main = new ZipFileSet());
     }
+
+    /** Fileset for platform/modules/ext/updater.jar, to be used in DTD validation. */
+    public Path createUpdaterJar() {
+        return updaterJar = new Path(getProject());
+    }
     
     private Attributes getModuleAttributesForLocale(String locale) throws BuildException {
         if (locale == null) {
             throw new BuildException("Unknown locale: null",getLocation());
         }
         log("Processing module attributes for locale '"+locale+"'", Project.MSG_VERBOSE);
-        Attributes attr = null;
+        Attributes attr;
         if ((!locale.equals("")) && (englishAttr != null)) {
             attr = new Attributes(englishAttr);
             attr.putValue("locale", locale);
@@ -465,7 +501,7 @@ public class MakeNBM extends Task {
         }
         log("Going to open jarfile "+jarName,Project.MSG_VERBOSE);
         File mfile = new File(productDir, jarName );
-        if ((mfile == null) || (!mfile.exists())) {
+        if (!mfile.exists()) {
             // localizing jarfile does not exist, try to return english data
             if (englishAttr != null) {
                 Attributes xattr = new Attributes(englishAttr);
@@ -478,6 +514,10 @@ public class MakeNBM extends Task {
         try {
             JarFile mjar = new JarFile(mfile);
             try {
+                if (mjar.getManifest().getMainAttributes().getValue("Bundle-SymbolicName") != null) {
+                    // #181025: treat bundles specially.
+                    return null;
+                }
                 if (attr.getValue("OpenIDE-Module") == null ) {
                     attr = mjar.getManifest().getMainAttributes();
                     attr.putValue("locale", locale);
@@ -555,12 +595,12 @@ public class MakeNBM extends Task {
             locales = new ArrayList<String>();
         }
 
-    File file;
+    File nbm;
     String rootDir = getProject ().getProperty ("nbm.target.dir");
     if (rootDir != null && !rootDir.equals ("")) { 
-        file = new File (rootDir, this.file.getName ());
+        nbm = new File(rootDir, this.file.getName());
     } else {
-        file = this.file;
+        nbm = this.file;
     }
 
 	// If desired, override the license and/or URL. //
@@ -569,25 +609,36 @@ public class MakeNBM extends Task {
         
         
         moduleAttributes = new ArrayList<Attributes> ();
-        moduleAttributes.add(getModuleAttributesForLocale(""));
+        File module = new File( productDir, moduleName );
+        Attributes attr = getModuleAttributesForLocale("");
+        if (attr == null) {
+            // #181025: OSGi bundle, copy unmodified.
+            Copy copy = new Copy();
+            copy.setProject(getProject());
+            copy.setOwningTarget(getOwningTarget());
+            copy.setFile(module);
+            copy.setTofile(new File(nbm.getAbsolutePath().replaceFirst("[.]nbm$", ".jar")));
+            copy.execute();
+            // XXX possibly sign it
+            // XXX could try to run pack200, though not if it was signed
+            return;
+        }
+        moduleAttributes.add(attr);
         for (String locale : locales) {
             Attributes a = getModuleAttributesForLocale(locale);
             if (a != null) moduleAttributes.add(a);
         }
 
-        File module = new File( productDir, moduleName );
         // Will create a file Info/info.xml to be stored in tmp
-        if (module != null) {
             // The normal case; read attributes from its manifest and maybe bundle.
             long mMod = module.lastModified();
             if (mostRecentInput < mMod) mostRecentInput = mMod;
-        }
 
-        if (mostRecentInput < file.lastModified()) {
-            log("Skipping NBM creation as most recent input is younger: " + mostRecentInput + " than the target file: " + file.lastModified(), Project.MSG_VERBOSE);
+        if (mostRecentInput < nbm.lastModified()) {
+            log("Skipping NBM creation as most recent input is younger: " + mostRecentInput + " than the target file: " + nbm.lastModified(), Project.MSG_VERBOSE);
             return;
         } else {
-            log("Most recent input: " + mostRecentInput + " file: " + file.lastModified(), Project.MSG_DEBUG);
+            log("Most recent input: " + mostRecentInput + " file: " + nbm.lastModified(), Project.MSG_DEBUG);
         }
         
         ArrayList<ZipFileSet> infoXMLFileSets = new ArrayList<ZipFileSet>();
@@ -626,17 +677,78 @@ public class MakeNBM extends Task {
  	UpdateTracking tracking = new UpdateTracking(productDir.getAbsolutePath());
  	String files[] = tracking.getListOfNBM( codename );
  	ZipFileSet fs = new ZipFileSet();
+        List <String> moduleFiles = new ArrayList <String>();
  	fs.setDir( productDir );
- 	for (int i=0; i < files.length; i++)
- 	    fs.createInclude().setName( files[i] );
+        String [] filesForPackaging = null;
+        if(usePack200 && pack200excludes!=null && !pack200excludes.equals("")) {
+            FileSet pack200Files = new FileSet();
+            pack200Files.setDir(productDir);
+            pack200Files.setExcludes(pack200excludes);
+            pack200Files.setProject(getProject());
+            for (int i=0; i < files.length; i++) {
+                  pack200Files.createInclude().setName( files[i] );
+            }
+            DirectoryScanner ds = pack200Files.getDirectoryScanner();
+            ds.scan();
+            filesForPackaging = ds.getIncludedFiles();            
+        }
+
+        List<File> packedFiles = new ArrayList<File>();
+        for (int i = 0; i < files.length; i++) {
+            if (usePack200) {
+                File sourceFile = new File(productDir, files[i]);
+                if (sourceFile.isFile() && sourceFile.getName().endsWith(".jar")) {
+
+                    boolean doPackage = true;
+                    if (filesForPackaging != null) {
+                        doPackage = false;
+                        for (String f : filesForPackaging) {
+                            if (new File(productDir, f).equals(sourceFile)) {
+                                doPackage = true;
+                                break;
+                            }
+                        }
+                    }
+                    if(doPackage) {
+                        //if both <filename>.jar and <filename>.jad exist - skip it
+                        //if both <filename>.jar and <filename>.jar.pack.gz exist - skip it                        
+                        for (String f : files) {
+                            if(f.equals(files[i].substring(0, files[i].lastIndexOf(".jar")) + ".jad") ||
+                                    f.equals(files[i] + ".pack.gz")) {
+                                doPackage = false;
+                                break;
+                            }
+                        }
+
+                    }
+                    if (doPackage) {
+                        File targetFile = new File(productDir, files[i] + ".pack.gz");
+                        try {
+                            if (pack200(sourceFile, targetFile)) {
+                                packedFiles.add(targetFile);
+                                files[i] = files[i] + ".pack.gz";
+                            }
+                        } catch (IOException e) {
+                            if(targetFile.exists()) {
+                                targetFile.delete();
+                            }
+                            log("Cannot pack file " + sourceFile, e, Project.MSG_WARN);
+                        }
+                    }
+                }
+            }
+
+            fs.createInclude().setName(files[i]);
+            moduleFiles.add(files[i]);
+        }
  	fs.setPrefix("netbeans/");
 
-	// JAR it all up together.
-	long jarModified = file.lastModified (); // may be 0
+        // JAR it all up together.
+        long jarModified = nbm.lastModified(); // may be 0
 	//log ("Ensuring existence of NBM file " + file);
 	Jar jar = (Jar) getProject().createTask("jar");
     
-        jar.setDestFile(file);
+        jar.setDestFile(nbm);
         jar.addZipfileset(fs);
         for (ZipFileSet zfs : infoXMLFileSets) {
             jar.addFileset(zfs);
@@ -645,15 +757,59 @@ public class MakeNBM extends Task {
         if (main != null) { // Add the main dir
             main.setPrefix("main"); // use main prefix
             jar.addZipfileset(main);
+            DirectoryScanner ds = main.getDirectoryScanner();
+            ds.scan();
+            String  [] mainFiles = ds.getIncludedFiles();
+            for(String m : mainFiles) {
+                moduleFiles.add(m);
+            }
         }
-            
+
+
+        if (executablesSet != null) {
+            DirectoryScanner eds = executablesSet.getDirectoryScanner();
+            eds.scan();
+            String  [] executables = eds.getIncludedFiles();
+
+            if(executables.length > 0) {
+                ZipFileSet executablesList = new ZipFileSet();
+                File executablesFile;
+                StringBuilder sb = new StringBuilder("");
+                    String ls = System.getProperty("line.separator");
+                    for(int i=0;i < executables.length;i++) {
+                        if(i!=0) {
+                            sb.append(ls);
+                        }
+                        sb.append(executables[i].replace("\\","/"));
+                    }
+                try {
+                    executablesFile = File.createTempFile("executables",".list");
+                    OutputStream infoStream = new FileOutputStream (executablesFile);
+                    try {
+                        infoStream.write(sb.toString().getBytes("UTF-8"));
+                    } finally {
+                        infoStream.close ();
+                    }
+                } catch (IOException e) {
+                    throw new BuildException("exception when creating Info/executables.list", e, getLocation());
+                }
+                executablesFile.deleteOnExit();
+                executablesList.setFile(executablesFile);
+                executablesList.setFullpath("Info/executables.list");
+                jar.addZipfileset(executablesList);
+            }
+        }
+
         jar.setCompress(true);
 	jar.setLocation(getLocation());
 	jar.init ();
 	jar.execute ();
+        for(File f : packedFiles) {
+            f.delete();
+        }
 
 	// Print messages if we overrode anything. //
-	if( file.lastModified () != jarModified) {
+        if (nbm.lastModified() != jarModified) {
 	  if( overrideLicense()) {
 	    log( "Overriding license with: " + getLicenseOverride()) ;
 	  }
@@ -663,7 +819,7 @@ public class MakeNBM extends Task {
 	}
 
 	// Maybe sign it.
-	if (signature != null && file.lastModified () != jarModified) {
+        if (signature != null && nbm.lastModified() != jarModified) {
 	    if (signature.keystore == null)
 		throw new BuildException ("must define keystore attribute on <signature/>");
 	    if (signature.storepass == null)
@@ -671,10 +827,10 @@ public class MakeNBM extends Task {
 	    if (signature.alias == null)
 		throw new BuildException ("must define alias attribute on <signature/>");
             if (signature.storepass.equals ("?") || signature.storepass.indexOf("${") != -1 || !signature.keystore.exists()) {
-                log ("Not signing NBM file " + file + "; no stored-key password provided or keystore (" 
+                log("Not signing NBM file " + nbm + "; no stored-key password provided or keystore ("
 		     + signature.keystore.toString() + ") doesn't exist", Project.MSG_WARN);
             } else {
-                log ("Signing NBM file " + file);
+                log("Signing NBM file " + nbm);
                 SignJar signjar = (SignJar) getProject().createTask("signjar");
                 try { // Signatures changed in various Ant versions.
                     try {
@@ -683,9 +839,9 @@ public class MakeNBM extends Task {
                         SignJar.class.getMethod("setKeystore", String.class).invoke(signjar, signature.keystore.getAbsolutePath());
                     }
                     try {
-                        SignJar.class.getMethod("setJar", File.class).invoke(signjar, file);
+                        SignJar.class.getMethod("setJar", File.class).invoke(signjar, nbm);
                     } catch (NoSuchMethodException x) {
-                        SignJar.class.getMethod("setJar", String.class).invoke(signjar, file.getAbsolutePath());
+                        SignJar.class.getMethod("setJar", String.class).invoke(signjar, nbm.getAbsolutePath());
                     }
                 } catch (BuildException x) {
                     throw x;
@@ -702,6 +858,68 @@ public class MakeNBM extends Task {
 	}
     }
 
+    private boolean isSigned(final JarFile jar) throws IOException {
+        Enumeration<JarEntry> entries = jar.entries();
+        boolean signatureInfoPresent = false;
+        boolean signatureFilePresent = false;
+        while (entries.hasMoreElements()) {
+            String entryName = entries.nextElement().getName();
+            if (entryName.startsWith("META-INF/")) {
+                if (entryName.endsWith(".RSA") || entryName.endsWith(".DSA")) {
+                    signatureFilePresent = true;
+                    if (signatureInfoPresent) {
+                        break;
+                    }
+                } else if (entryName.endsWith(".SF")) {
+                    signatureInfoPresent = true;
+                    if (signatureFilePresent) {
+                        break;
+                    }
+                }
+            }
+        }
+        return signatureFilePresent && signatureInfoPresent;
+    }
+
+    public static boolean isWindows() {
+        String os = System.getProperty("os.name"); // NOI18N
+        return (os != null && os.toLowerCase().startsWith("windows"));//NOI18N
+    }
+
+    private boolean pack200(final File sourceFile, final File targetFile) throws IOException {
+        JarFile jarFile = new JarFile(sourceFile);
+        try {
+            if (isSigned(jarFile)) {
+                return false;
+            }
+
+            try {
+                String pack200Executable = new File(System.getProperty("java.home"),
+                        "bin/pack200" + (isWindows() ? ".exe" : "")).getAbsolutePath();
+
+                ProcessBuilder pb = new ProcessBuilder(
+                        pack200Executable,
+                        targetFile.getAbsolutePath(),
+                        sourceFile.getAbsolutePath());
+                
+                pb.directory(sourceFile.getParentFile());
+                int result;
+                Process process = pb.start();
+                result = process.waitFor();
+                process.destroy();
+                return result == 0;
+            } catch (InterruptedException e) {
+                return false;
+            } finally {
+                if (targetFile.exists())  {
+                    targetFile.setLastModified(sourceFile.lastModified());
+                }
+            }
+        } finally {
+            jarFile.close();
+        }
+    }
+
     private Document createInfoXml(final Attributes attr) throws BuildException {
         DOMImplementation domimpl;
         try {
@@ -717,7 +935,8 @@ public class MakeNBM extends Task {
         }
         
         String pub, sys;
-        if (attr.getValue("AutoUpdate-Show-In-Client") != null || attr.getValue("AutoUpdate-Essential-Module") != null) {
+        if (attr.getValue("AutoUpdate-Show-In-Client") != null || attr.getValue("AutoUpdate-Essential-Module") != null ||
+                attr.getValue("OpenIDE-Module-Recommends") != null || attr.getValue("OpenIDE-Module-Needs") != null) {
             pub = "-//NetBeans//DTD Autoupdate Module Info 2.5//EN";
             sys = "http://www.netbeans.org/dtds/autoupdate-info-2_5.dtd";
         } else if (targetcluster != null && !("".equals(targetcluster))) {
@@ -779,8 +998,8 @@ public class MakeNBM extends Task {
             releasedate = DATE_FORMAT.format(new Date(System.currentTimeMillis()));
         }
         module.setAttribute("releasedate", releasedate);
-        if (description != null) {
-            module.appendChild(doc.createElement("description")).appendChild(description.getTextNode(doc));
+        if (desc != null) {
+            module.appendChild(doc.createElement("description")).appendChild(desc.getTextNode(doc));
         }
         if (notification != null) {
             module.appendChild(doc.createElement("module_notification")).appendChild(notification.getTextNode(doc));
@@ -814,18 +1033,10 @@ public class MakeNBM extends Task {
         it = attrNames.iterator();
         while (it.hasNext()) {
             String name = (String) it.next();
-            // Ignore irrelevant attributes (cf. www/www/dtds/autoupdate-catalog-*.dtd
-            //  and www/www/dtds/autoupdate-info-*.dtd):
-            // XXX better would be to enumerate the attrs it *does* recognize!
-            if (!name.startsWith("OpenIDE-Module") && !name.startsWith("AutoUpdate-")) continue;
-            if (name.equals("OpenIDE-Module-Localizing-Bundle")) continue;
-            if (name.equals("OpenIDE-Module-Install")) continue;
-            if (name.equals("OpenIDE-Module-Layer")) continue;
-            if (name.equals("OpenIDE-Module-Description")) continue;
-            if (name.equals("OpenIDE-Module-Package-Dependency-Message")) continue;
-            if (name.equals("OpenIDE-Module-Public-Packages")) continue;
-            if (name.equals("OpenIDE-Module-Friends")) continue;
-            el.setAttribute(name, attr.getValue(name));
+            if (name.matches("OpenIDE-Module(|-(Name|(Specification|Implementation)-Version|(Module|Package|Java|IDE)-Dependencies|" +
+                    "(Short|Long)-Description|Display-Category|Provides|Requires|Recommends|Needs))|AutoUpdate-(Show-In-Client|Essential-Module)")) {
+                el.setAttribute(name, attr.getValue(name));
+            }
         }
         module.appendChild(el);
         // Maybe write out license text.
@@ -835,9 +1046,42 @@ public class MakeNBM extends Task {
             el.appendChild(license.getTextNode(doc));
             module.appendChild(el);
         }
+        if (updaterJar != null && updaterJar.size() > 0) {
+            try {
+                ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                XMLUtil.write(doc, baos);
+                validateAgainstAUDTDs(new InputSource(new ByteArrayInputStream(baos.toByteArray())), updaterJar, this);
+            } catch (Exception x) {
+                throw new BuildException("Could not validate Info.xml before writing: " + x, x, getLocation());
+            }
+        } else {
+            log("No updater.jar specified, cannot validate Info.xml against DTD", Project.MSG_WARN);
+        }
         return doc;
     }
-   
+
+    static void validateAgainstAUDTDs(InputSource input, final Path updaterJar, final Task task) throws IOException, SAXException {
+        XMLUtil.parse(input, true, false, new ErrorHandler() {
+            public void warning(SAXParseException exception) throws SAXException {throw exception;}
+            public void error(SAXParseException exception) throws SAXException {throw exception;}
+            public void fatalError(SAXParseException exception) throws SAXException {throw exception;}
+        }, new EntityResolver() {
+            ClassLoader loader = new AntClassLoader(task.getProject(), updaterJar);
+            public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
+                String remote = "http://www.netbeans.org/dtds/";
+                if (systemId.startsWith(remote)) {
+                    String rsrc = "org/netbeans/updater/resources/" + systemId.substring(remote.length());
+                    URL u = loader.getResource(rsrc);
+                    if (u != null) {
+                        return new InputSource(u.toString());
+                    } else {
+                        task.log(rsrc + " not found in " + updaterJar, Project.MSG_WARN);
+                    }
+                }
+                return null;
+            }
+        });
+    }
 
   /** This returns true if the license should be overridden. */
   protected boolean overrideLicense() {
