@@ -1,12 +1,12 @@
 /*
- * Copyright 2007-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
- * published by the Free Software Foundation.  Sun designates this
+ * published by the Free Software Foundation.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the LICENSE file that accompanied this code.
+ * by Oracle in the LICENSE file that accompanied this code.
  *
  * This code is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -18,9 +18,9 @@
  * 2 along with this work; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
- * CA 95054 USA or visit www.sun.com if you need additional information or
- * have any questions.
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
  */
 package com.sun.tools.visualvm.core.ui.components;
 
@@ -200,7 +200,10 @@ class DisplayArea extends JComponent {
             public void hierarchyChanged(HierarchyEvent e) {
                 if ((e.getChangeFlags() & HierarchyEvent.SHOWING_CHANGED) != 0) {
                     boolean isShowing = isShowing();
-                    presenter.setToolTipText(isShowing ? "Hide " + caption : "Show " + caption);
+                    presenter.setToolTipText(isShowing ? NbBundle.getMessage(
+                            DisplayArea.class, "ToolTip_HideSection", caption) : // NOI18N
+                            NbBundle.getMessage(DisplayArea.class,
+                            "ToolTip_ShowSection", caption)); // NOI18N
                     if (!internalChange[0]) presenter.setSelected(isShowing);
                 }
             }
@@ -229,7 +232,7 @@ class DisplayArea extends JComponent {
         add(viewContainer, BorderLayout.CENTER);
     }
 
-    private String caption = "";
+    private String caption = ""; // NOI18N
     private Presenter presenter;
 
     private ViewArea viewContainer;
@@ -279,8 +282,8 @@ class DisplayArea extends JComponent {
         public void setView(JComponent view) { this.view = view; }
         public JComponent getView() { return view; }
 
-        public void setOptions(JComponent[] options) { this.options = options; }
-        public JComponent[] getOptions() { return options; }
+        public void setOptions(JComponent[] options) { this.options = options != null ? options.clone() : null; }
+        public JComponent[] getOptions() { return options != null ? options.clone() : null; }
 
     }
 
