@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -40,6 +43,8 @@
 
 package org.netbeans.lib.profiler.classfile;
 
+import org.netbeans.lib.profiler.classfile.ClassInfo.LocalVariableTables;
+import org.netbeans.lib.profiler.classfile.ClassInfo.LocalVariableTypeTables;
 import org.netbeans.lib.profiler.instrumentation.JavaClassConstants;
 import org.netbeans.lib.profiler.utils.StringUtils;
 
@@ -48,8 +53,10 @@ import org.netbeans.lib.profiler.utils.StringUtils;
  * This class implements parsing a byte array representing a class file, generating a ClassInfo object.
  *
  * @author Misha Dmitirev
+ * @author Tomas Hurka
  */
 public class ClassFileParser implements JavaClassConstants {
+
     //~ Inner Classes ------------------------------------------------------------------------------------------------------------
 
     public static class ClassFileReadException extends Exception {
@@ -371,7 +378,7 @@ public class ClassFileParser implements JavaClassConstants {
 
         classInfo.intermediateDataStartOfs = curBufPos;
         classInfo.accessFlags = nextChar();
-        classIdx = nextChar();
+        classInfo.classIndex = classIdx = nextChar();
 
         if (cpTags[classIdx] != CONSTANT_Class) {
             throw classFileReadException("Bad reference to this class name"); // NOI18N
@@ -428,7 +435,16 @@ public class ClassFileParser implements JavaClassConstants {
         int[] exceptionTableStartOffsets = new int[methodCount];
         int[] lineNumberTableOffsets = new int[methodCount];
         char[] lineNumberTableLengths = new char[methodCount];
-
+        int[] localVariableTableOffsets = new int[methodCount];
+        char[] localVariableTableLengths = new char[methodCount];
+        int localVaribaleTableCPindex = 0;
+        int[] localVariableTypeTableOffsets = new int[methodCount];
+        char[] localVariableTypeTableLengths = new char[methodCount];
+        int localVaribaleTypeTableCPindex = 0;
+        int[] stackMapTableOffsets = new int[methodCount];
+        char[] stackMapTableLengths = new char[methodCount];
+        int stackMapTableCPindex = 0;
+        
         for (int i = 0; i < methodCount; i++) {
             methodInfoOffsets[i] = curBufPos;
             accessFlags[i] = nextChar();
@@ -436,7 +452,9 @@ public class ClassFileParser implements JavaClassConstants {
             signatures[i] = signatureAtCPIndex(nextChar());
             bytecodeOffsets[i] = 0;
             lineNumberTableOffsets[i] = 0;
-
+            localVariableTableOffsets[i] = 0;
+            localVariableTypeTableOffsets[i] = 0;
+            
             int attrCount = nextChar();
 
             for (int j = 0; j < attrCount; j++) {
@@ -460,10 +478,37 @@ public class ClassFileParser implements JavaClassConstants {
                         attrLen = nextInt();
 
                         if (utf8AtCPIndex(attrNameIdx).equals("LineNumberTable")) { // NOI18N
-
                             char tableLen = lineNumberTableLengths[i] = nextChar();
                             lineNumberTableOffsets[i] = curBufPos - methodInfoOffsets[i];
                             curBufPos += (4 * tableLen);
+                        } else if (utf8AtCPIndex(attrNameIdx).equals("LocalVariableTable")){    // NOI18N
+                            char tableLen = localVariableTableLengths[i] = nextChar();
+                            localVariableTableOffsets[i] = curBufPos - methodInfoOffsets[i];
+                            curBufPos += LocalVariableTables.ATTR_SIZE * tableLen;
+                            if (localVaribaleTableCPindex == 0) {
+                                localVaribaleTableCPindex = attrNameIdx;
+                            } else {
+                                assert localVaribaleTableCPindex == attrNameIdx;
+                            }
+                        } else if (utf8AtCPIndex(attrNameIdx).equals("LocalVariableTypeTable")){    // NOI18N
+                            char tableLen = localVariableTypeTableLengths[i] = nextChar();
+                            localVariableTypeTableOffsets[i] = curBufPos - methodInfoOffsets[i];
+                            curBufPos += LocalVariableTypeTables.ATTR_SIZE * tableLen;
+                            if (localVaribaleTypeTableCPindex == 0) {
+                                localVaribaleTypeTableCPindex = attrNameIdx;
+                            } else {
+                                assert localVaribaleTypeTableCPindex == attrNameIdx;
+                            }
+                        } else if (utf8AtCPIndex(attrNameIdx).equals("StackMapTable")){    // NOI18N
+                            char tableLen = stackMapTableLengths[i] = nextChar();
+                            stackMapTableOffsets[i] = curBufPos - methodInfoOffsets[i];
+                            curBufPos += attrLen - 2;
+                            if (stackMapTableCPindex == 0) {
+                                stackMapTableCPindex = attrNameIdx;
+                            } else {
+                                assert stackMapTableCPindex == attrNameIdx;
+                            }
+//                            LOG.info("StackMapTable size:"+(tableLen+0)+" for class:"+classInfo.name+" method:"+ names[i]);
                         } else {
                             curBufPos += attrLen;
                         }
@@ -486,6 +531,15 @@ public class ClassFileParser implements JavaClassConstants {
         classInfo.exceptionTableStartOffsets = exceptionTableStartOffsets;
         classInfo.lineNumberTablesOffsets = lineNumberTableOffsets;
         classInfo.lineNumberTablesLengths = lineNumberTableLengths;
+        classInfo.localVariableTablesOffsets = localVariableTableOffsets;
+        classInfo.localVariableTablesLengths = localVariableTableLengths;
+        classInfo.localVaribaleTableCPindex = localVaribaleTableCPindex;
+        classInfo.localVariableTypeTablesOffsets = localVariableTypeTableOffsets;
+        classInfo.localVariableTypeTablesLengths = localVariableTypeTableLengths;
+        classInfo.localVaribaleTypeTableCPindex = localVaribaleTypeTableCPindex;
+        classInfo.stackMapTablesOffsets = stackMapTableOffsets;
+        classInfo.stackMapTablesLengths = stackMapTableLengths;
+        classInfo.stackMapTableCPindex = stackMapTableCPindex;
     }
 
     private void readPreamble() {
@@ -500,12 +554,13 @@ public class ClassFileParser implements JavaClassConstants {
 
         if ((majorVersion > JAVA_MAJOR_VERSION)
                 || (((majorVersion * 1000) + minorVersion) < ((JAVA_MIN_MAJOR_VERSION * 1000) + JAVA_MIN_MINOR_VERSION))) {
-            if ((majorVersion != 49) && (majorVersion != 50)) {
+            if ((majorVersion != 49) && (majorVersion != 50) && (majorVersion != 51)) {
                 String versionCode = majorVersion + "." + minorVersion; // NOI18N
                 String message = "Unsupported class file version: " + versionCode; // NOI18N
                 throw classFileReadException(message);
             }
         }
+        classInfo.majorVersion = majorVersion;
     }
 
     private String signatureAtCPIndex(int idx) {
