@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -58,6 +61,7 @@ public class DynamicClassInfo extends ClassInfo {
     private DynamicClassInfo superClass; // Superclass as a DynamicClassInfo (just name is inconvenient when multiple classloaders are used)
     private String classFileLocation; // Directory or .jar file where the .class file is located.
     private int[] baseCPoolCount;
+    private int java_lang_ThowableCPIndex; // constant pool index for java.lang.Throwable
     private char[] instrMethodIds; // Ids assigned to instrumented methods, 0 for uninstrumented methods
     private DynamicClassInfo[] interfacesDCI; // Ditto for superinterfaces
 
@@ -69,6 +73,10 @@ public class DynamicClassInfo extends ClassInfo {
     // same class are instrumented one-by-one and redefineClasses is used for each of them, we don't have to regenerate the
     // code for each previously instrumented method over and over again.
     private byte[][] modifiedAndSavedMethodInfos;
+    private int[] modifiedMethodBytecodesLength;
+    private int[] modifiledLocalVariableTableOffsets;
+    private int[] modifiledLocalVariableTypeTableOffsets;
+    private int[] modifiledStackMapTableOffsets;
     private boolean allMethodsMarkers = false;
     private boolean allMethodsRoots = false;
     private boolean hasUninstrumentedMarkerMethods;
@@ -172,6 +180,78 @@ public class DynamicClassInfo extends ClassInfo {
         }
     }
 
+    public int getLocalVariableTableStartOffsetInMethodInfo(int idx) {
+        if ((modifiedAndSavedMethodInfos != null) && (modifiedAndSavedMethodInfos[idx] != null)) {
+            if (modifiledLocalVariableTableOffsets[idx] == 0) {
+                int newOffset = getExceptionTableStartOffsetInMethodInfo(idx)+getExceptionTableCount(idx)*8+2;
+                byte[] methodInfo = getMethodInfo(idx);
+                int attrCount = getU2(methodInfo, newOffset); newOffset+=2;// Attribute (or rather sub-attribute) count
+
+                for (int k = 0; k < attrCount; k++) {
+                    int attrNameIdx = getU2(methodInfo, newOffset); newOffset+=2;
+                    int attrLen = getU4(methodInfo, newOffset); newOffset+=4;
+
+                    if (attrNameIdx==localVaribaleTableCPindex){
+                        modifiledLocalVariableTableOffsets[idx] = newOffset+2;
+                        break;
+                    }
+                    newOffset += attrLen;
+                }
+            }
+            return modifiledLocalVariableTableOffsets[idx];
+        } else {
+            return super.getLocalVariableTableStartOffsetInMethodInfo(idx);
+        }
+    }
+    
+    public int getLocalVariableTypeTableStartOffsetInMethodInfo(int idx) {
+        if ((modifiedAndSavedMethodInfos != null) && (modifiedAndSavedMethodInfos[idx] != null)) {
+            if (modifiledLocalVariableTypeTableOffsets[idx] == 0) {
+                int newOffset = getExceptionTableStartOffsetInMethodInfo(idx)+getExceptionTableCount(idx)*8+2;
+                byte[] methodInfo = getMethodInfo(idx);
+                int attrCount = getU2(methodInfo, newOffset); newOffset+=2;// Attribute (or rather sub-attribute) count
+
+                for (int k = 0; k < attrCount; k++) {
+                    int attrNameIdx = getU2(methodInfo, newOffset); newOffset+=2;
+                    int attrLen = getU4(methodInfo, newOffset); newOffset+=4;
+
+                    if (attrNameIdx==localVaribaleTypeTableCPindex){
+                        modifiledLocalVariableTypeTableOffsets[idx] = newOffset+2;
+                        break;
+                    }
+                    newOffset += attrLen;
+                }
+            }
+            return modifiledLocalVariableTypeTableOffsets[idx];
+        } else {
+            return super.getLocalVariableTypeTableStartOffsetInMethodInfo(idx);
+        }
+    }
+    
+    public int getStackMapTableStartOffsetInMethodInfo(int idx) {
+        if ((modifiedAndSavedMethodInfos != null) && (modifiedAndSavedMethodInfos[idx] != null)) {
+            if (modifiledStackMapTableOffsets[idx] == 0) {
+                int newOffset = getExceptionTableStartOffsetInMethodInfo(idx)+getExceptionTableCount(idx)*8+2;
+                byte[] methodInfo = getMethodInfo(idx);
+                int attrCount = getU2(methodInfo, newOffset); newOffset+=2;// Attribute (or rather sub-attribute) count
+
+                for (int k = 0; k < attrCount; k++) {
+                    int attrNameIdx = getU2(methodInfo, newOffset); newOffset+=2;
+                    int attrLen = getU4(methodInfo, newOffset); newOffset+=4;
+
+                    if (attrNameIdx==stackMapTableCPindex){
+                        modifiledStackMapTableOffsets[idx] = newOffset+2;
+                        break;
+                    }
+                    newOffset += attrLen;
+                }
+            }
+            return modifiledStackMapTableOffsets[idx];
+        } else {
+            return super.getStackMapTableStartOffsetInMethodInfo(idx);
+        }
+    }
+    
     public void setHasUninstrumentedMarkerMethods(boolean v) {
         hasUninstrumentedMarkerMethods = v;
     }
@@ -433,6 +513,10 @@ public class DynamicClassInfo extends ClassInfo {
         }
 
         modifiedAndSavedMethodInfos[idx] = methodInfo;
+        modifiedMethodBytecodesLength = new int[methodNames.length];
+        modifiledLocalVariableTableOffsets = new int[methodNames.length];
+        modifiledLocalVariableTypeTableOffsets = new int[methodNames.length];
+        modifiledStackMapTableOffsets = new int[methodNames.length];
     }
 
     public void unsetMethodInstrumented(int i) {
@@ -444,11 +528,53 @@ public class DynamicClassInfo extends ClassInfo {
         methodScanStatus[i] &= (~128);
     }
 
-    private int getBCLenForModifiedAndSavedMethodInfo(int idx) {
-        byte[] methodInfo = modifiedAndSavedMethodInfos[idx];
-        int bcLenPos = methodBytecodesOffsets[idx] - 4;
+    public void addGlobalCatchStackMapTableEntry(int methodIdx, int endPC) {
+        if (majorVersion >= 50) {
+            boolean isStatic = isMethodStatic(methodIdx);
+            boolean constructor = "<init>".equals(getMethodName(methodIdx));    // NOI18N
+            int[] localsCPIdx = new int[0];
+            int[] stacksCPIdx;
+            StackMapTables tables = getStackMapTables();
+            
+//            LOG.finer("Adding global catch for " + getName() + " method " + getMethodName(methodIdx));   // NOI18N
+            if (stackMapTableCPindex == 0) {
+                stackMapTableCPindex = getBaseCPoolCount(INJ_STACKMAP);
+            }
+            if (java_lang_ThowableCPIndex == 0) {
+                java_lang_ThowableCPIndex = getCPIndexOfClass("java/lang/Throwable");   // NOI18N
+                if (java_lang_ThowableCPIndex == -1) {
+//                    LOG.finer("java/lang/Thowable not found in " + getName());   // NOI18N
+                    java_lang_ThowableCPIndex = getBaseCPoolCount(INJ_THROWABLE);
+                }
+            }
+            stacksCPIdx = new int[] {java_lang_ThowableCPIndex};
+            if (!isStatic) {
+                if (constructor) {
+                    localsCPIdx = new int[] {0};
+                } else {
+                    localsCPIdx = new int[] {classIndex};
+                }
+            }
+            getStackMapTables().addFullStackMapFrameEntry(methodIdx, endPC, localsCPIdx, stacksCPIdx);
+        }
+    }
 
-        return (((methodInfo[bcLenPos++] & 255) << 24) + ((methodInfo[bcLenPos++] & 255) << 16)
-               + ((methodInfo[bcLenPos++] & 255) << 8) + (methodInfo[bcLenPos++] & 255));
+    private int getBCLenForModifiedAndSavedMethodInfo(int idx) {
+        if (modifiedMethodBytecodesLength[idx] == 0) {
+            byte[] methodInfo = modifiedAndSavedMethodInfos[idx];
+            int bcLenPos = methodBytecodesOffsets[idx] - 4;
+
+            modifiedMethodBytecodesLength[idx] =  getU4(methodInfo,bcLenPos);
+        }
+        return modifiedMethodBytecodesLength[idx];
+    }
+
+    static int getU2(byte[] bytecodes, int pos) {
+        return ((bytecodes[pos] & 0xFF) << 8) + (bytecodes[pos + 1] & 0xFF);
+    }
+
+    static int getU4(byte[] bytecodes, int pos) {
+        return ((bytecodes[pos] & 0xFF) << 24) + ((bytecodes[pos + 1] & 0xFF) << 16) + ((bytecodes[pos + 2] & 0xFF) << 8)
+               + (bytecodes[pos + 3] & 0xFF);
     }
 }
