@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -40,6 +43,7 @@
 
 package org.netbeans.lib.profiler.common;
 
+import java.awt.EventQueue;
 import org.netbeans.lib.profiler.ProfilerClient;
 import org.netbeans.lib.profiler.TargetAppRunner;
 import org.netbeans.lib.profiler.client.ClientUtils;
@@ -53,11 +57,10 @@ import org.netbeans.lib.profiler.instrumentation.BadLocationException;
 import org.netbeans.lib.profiler.instrumentation.InstrumentationException;
 import org.netbeans.lib.profiler.results.monitor.VMTelemetryDataManager;
 import org.netbeans.lib.profiler.results.threads.ThreadsDataManager;
-import sun.misc.Service;
 import java.io.IOException;
 import java.util.Iterator;
 import java.util.Vector;
-import javax.swing.*;
+import org.openide.util.Lookup;
 
 
 /** An abstract superclass representing the entire Profiler.  The Profiler class should add a "state" on top of the
@@ -117,21 +120,13 @@ public abstract class Profiler {
 
     public static synchronized Profiler getDefault() {
         if (defaultProfiler == null) {
-            final Iterator it = Service.providers(Profiler.class);
-
-            if (it.hasNext()) {
-                defaultProfiler = (Profiler) it.next();
-
-                if ((defaultProfiler != null) && DEBUG) {
-                    System.err.println("Default Profiler succesfully installed: " + defaultProfiler); // NOI18N
-                }
-            }
-
+            defaultProfiler = (Profiler) Lookup.getDefault().lookup(Profiler.class);
             if (defaultProfiler == null) {
                 throw new InternalError("Should never happen"); // NOI18N
+            } else if (DEBUG) {
+                System.err.println("Default Profiler succesfully installed: " + defaultProfiler); // NOI18N
             }
         }
-
         return defaultProfiler;
     }
 
@@ -354,10 +349,10 @@ public abstract class Profiler {
             }
         };
 
-        if (SwingUtilities.isEventDispatchThread()) {
+        if (EventQueue.isDispatchThread()) {
             r.run();
         } else {
-            SwingUtilities.invokeLater(r);
+            EventQueue.invokeLater(r);
         }
     }
 
@@ -388,10 +383,10 @@ public abstract class Profiler {
             }
         };
 
-        if (SwingUtilities.isEventDispatchThread()) {
+        if (EventQueue.isDispatchThread()) {
             r.run();
         } else {
-            SwingUtilities.invokeLater(r);
+            EventQueue.invokeLater(r);
         }
     }
 
@@ -415,10 +410,10 @@ public abstract class Profiler {
             }
         };
 
-        if (SwingUtilities.isEventDispatchThread()) {
+        if (EventQueue.isDispatchThread()) {
             r.run();
         } else {
-            SwingUtilities.invokeLater(r);
+            EventQueue.invokeLater(r);
         }
     }
 }
