@@ -46,6 +46,8 @@ package org.netbeans.lib.profiler.instrumentation;
 import org.netbeans.lib.profiler.classfile.ClassInfo;
 import org.netbeans.lib.profiler.client.RuntimeProfilingPoint;
 import java.util.Stack;
+import org.netbeans.lib.profiler.classfile.DynamicClassInfo;
+import org.netbeans.lib.profiler.global.CommonConstants;
 
 
 /**
@@ -54,6 +56,7 @@ import java.util.Stack;
  * supports appending bytecodes to the existing bytecodes, and extending the method's exception table.
  *
  * @author Misha Dmitriev
+ * @author Tomas Hurka
  */
 public abstract class Injector extends SingleMethodScaner {
     //~ Inner Classes ------------------------------------------------------------------------------------------------------------
@@ -153,6 +156,7 @@ public abstract class Injector extends SingleMethodScaner {
     protected int origExcTableEntryCount;
     private boolean changeTypeIsInjectNewInstr;
     private boolean injectionBindsToFollowingInstruction;
+    private boolean classChecked;
 
     //~ Constructors -------------------------------------------------------------------------------------------------------------
 
@@ -164,7 +168,7 @@ public abstract class Injector extends SingleMethodScaner {
      * Creates a new Injector for the specified method. Subsequently one can call injectCodeAndRewrite() and other
      * methods below as many times as needed.
      */
-    protected Injector(ClassInfo clazz, int methodIdx) {
+    protected Injector(DynamicClassInfo clazz, int methodIdx) {
         super(clazz, methodIdx);
         origBytecodesLength = bytecodesLength;
         maxStack = getU2(origMethodInfo, bytecodesStartIdx - 8);
@@ -189,6 +193,20 @@ public abstract class Injector extends SingleMethodScaner {
         excTableEntryCount++;
     }
 
+    protected void addGlobalCatchStackMapTableEntry(int endPC) {
+        DynamicClassInfo dynClass = (DynamicClassInfo)clazz;
+        if (!classChecked) {
+            classChecked = true;
+            if (!clazz.getStackMapTables().hasTable()) {
+                DynamicConstantPoolExtension.getCPFragment(dynClass, CommonConstants.INJ_STACKMAP);
+            }
+            if (clazz.getCPIndexOfClass("java/lang/Throwable") == -1) { // NOI18N
+                DynamicConstantPoolExtension.getCPFragment(dynClass, CommonConstants.INJ_THROWABLE);
+            }
+        }
+        dynClass.addGlobalCatchStackMapTableEntry(methodIdx, endPC);
+    }
+
     protected void appendCode(byte[] appendedBytes, int appendedBytesCount) {
         System.arraycopy(appendedBytes, 0, bytecodes, bytecodesLength, appendedBytesCount);
         bytecodesLength += appendedBytesCount;
@@ -220,13 +238,83 @@ public abstract class Injector extends SingleMethodScaner {
 
         System.arraycopy(exceptionTable, 0, ret, excTableNewStart + 2, excTableNewLen);
 
-        // FIXME: need to update linenumber table and localvariable table as well
+        ClassInfo.LocalVariableTables localVarTable = clazz.getLocalVariableTables();
+        
+        if (localVarTable.hasTable()) {
+            int locVarTableOldStart = clazz.getLocalVariableTableStartOffsetInMethodInfo(methodIdx);
+            int locVarTablePtr = locVarTableOldStart + diff;
+            localVarTable.writeTable(ret, locVarTablePtr, methodIdx);
+        }
+
+        ClassInfo.LocalVariableTypeTables localVarTypeTable = clazz.getLocalVariableTypeTables();
+        
+        if (localVarTypeTable.hasTable()) {
+            int locVarTypeTableOldStart = clazz.getLocalVariableTypeTableStartOffsetInMethodInfo(methodIdx);
+            int locVarTypeTablePtr = locVarTypeTableOldStart + diff;
+            localVarTypeTable.writeTable(ret, locVarTypeTablePtr, methodIdx);
+        }
+
+        ClassInfo.StackMapTables stackMapTable = clazz.getStackMapTables();
+        
+        if (stackMapTable.hasTable()) {
+            int stackMapTableOldStart = clazz.getStackMapTableStartOffsetInMethodInfo(methodIdx);
+            byte[] stackMapData = stackMapTable.writeTable(methodIdx);
+            
+            if (stackMapData != null) {
+                int stackMapDiff = 0;
+                int stackMapTablePtr;
+                
+                if (stackMapTableOldStart != 0) {
+//                    ClassInfo.LOG.finer("Updating StackMapTable for "+clazz.getName()+" method "+clazz.getMethodName(methodIdx));
+                    stackMapTablePtr = stackMapTableOldStart + diff;
+                    int originalSize = getU4(ret, stackMapTablePtr-6) - 2;
+//                    ClassInfo.LOG.finer("Original size "+originalSize);
+                    
+                    if (stackMapData.length > originalSize) {  // add space and copy original data
+                        stackMapDiff = stackMapData.length-originalSize;
+                        ret = insertBytes(ret, stackMapTablePtr+originalSize, stackMapDiff);
+                    }
+                    System.arraycopy(stackMapData,0,ret,stackMapTablePtr,stackMapData.length);
+                } else {
+//                    ClassInfo.LOG.finer("Adding StackMapTable for "+clazz.getName()+" method "+clazz.getMethodName(methodIdx));
+                    // add whole StackMapTable attribute
+                    byte[] header = stackMapTable.getAttributeHeader(methodIdx);
+                    int insertOffset = bytecodesStartIdx - 8 + attrLength; // end of Code attribute
+                    
+                    stackMapDiff = header.length + stackMapData.length;
+                    stackMapTablePtr = insertOffset + header.length;
+                    ret = insertBytes(ret, insertOffset, stackMapDiff);
+                    System.arraycopy(header, 0, ret, stackMapTablePtr-header.length , header.length);
+                    System.arraycopy(stackMapData, 0, ret, stackMapTablePtr, stackMapData.length);
+                    int arrtCountPrt = excTableNewStart+2+excTableNewLen;
+                    putU2(ret,arrtCountPrt,getU2(ret,arrtCountPrt)+1); // increment attributes_count item
+                }
+                if (stackMapDiff > 0) {
+                    putU4(ret, bytecodesStartIdx-12, attrLength + stackMapDiff);                  // update the attribute_length in Code attribute
+                    putU4(ret, stackMapTablePtr-6, stackMapData.length + 2);                          // update attribute size
+                    putU2(ret, stackMapTablePtr-2, stackMapTable.getNumberOfFrames(methodIdx));       // update no. of StackMap frames
+                }
+            }
+        }
+        
+        // FIXME: need to update linenumber table as well
         putU2(ret, bytecodesStartIdx - 8, maxStack + STACK_INCREMENT);
         putU2(ret, bytecodesStartIdx - 6, maxLocals);
 
         return ret;
     }
 
+    private byte[] insertBytes(final byte[] ret, final int insertionOffset, final int newBytes) {
+        byte[] newRet = new byte[ret.length+newBytes];
+        System.arraycopy(ret,0,newRet,0,insertionOffset);
+        if (insertionOffset < ret.length) {
+            System.arraycopy(ret, insertionOffset, 
+                             newRet, insertionOffset+newBytes,
+                             ret.length-insertionOffset);
+        }
+        return newRet;
+    }
+
     /**
      * Injects the specified raw bytes array at the specified location.
      * The last parameter indicates whether the injected code should be bound to the following (if true) or previous (if false) bytecode
@@ -496,7 +584,7 @@ public abstract class Injector extends SingleMethodScaner {
 
     private void initExceptionTable() {
         int startOfs = clazz.getExceptionTableStartOffsetInMethodInfo(methodIdx);
-        excTableEntryCount = getU2(origMethodInfo, startOfs);
+        excTableEntryCount = clazz.getExceptionTableCount(methodIdx);
 
         int len = excTableEntryCount * 8;
 
@@ -584,11 +672,12 @@ public abstract class Injector extends SingleMethodScaner {
         bytecodesLength += delta;
 
         updateExceptionTable(bci, delta);
+        updateLocalVariableTable(bci, delta);
+        updateLocalVariableTypeTable(bci, delta);
+        updateStackMapTable(bci, delta);
 
         // We currently don't support the following updates - they are used only by debuggers.
         // updateLineNumberTable(injectionPos, delta);
-        // updateLocalVariableTable(injectionPos, delta);
-        // updateLocalVariableTypeTable(injectionPos, delta);
 
         // Relocate the bcis of changes in the pending change stack
         for (int j = 0; j < changes.size(); j++) {
@@ -623,4 +712,22 @@ public abstract class Injector extends SingleMethodScaner {
             pos += 8;
         }
     }
+
+    private void updateLocalVariableTable(int injectionPos, int injectedBytesCount) {
+        ClassInfo.LocalVariableTables localVarTable = clazz.getLocalVariableTables();
+        
+        localVarTable.updateTable(injectionPos, injectedBytesCount, methodIdx);
+    }
+
+    private void updateLocalVariableTypeTable(int injectionPos, int injectedBytesCount) {
+        ClassInfo.LocalVariableTypeTables localVarTypeTable = clazz.getLocalVariableTypeTables();
+        
+        localVarTypeTable.updateTable(injectionPos, injectedBytesCount, methodIdx);
+    }
+
+    private void updateStackMapTable(int injectionPos, int injectedBytesCount) {
+        ClassInfo.StackMapTables stackMapTable = clazz.getStackMapTables();
+        
+        stackMapTable.updateTable(injectionPos, injectedBytesCount, methodIdx);
+    }
 }
