@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -118,7 +121,6 @@ import java.text.MessageFormat;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.Properties;
-import java.util.concurrent.Semaphore;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.swing.JLabel;
@@ -366,7 +368,7 @@ public final class NetBeansProfiler extends Profiler {
                 LiveResultsWindow.getDefault().handleShutdown();
             }
 
-            if ((Profiler.getDefault().getTargetAppRunner().getProfilerClient().getCurrentInstrType() == CommonConstants.INSTR_NONE)
+            if ((getTargetAppRunner().getProfilerClient().getCurrentInstrType() == CommonConstants.INSTR_NONE)
                     || !ResultsManager.getDefault().resultsAvailable()) {
                 ProfilerDialogs.DNSAMessage dnsa = new ProfilerDialogs.DNSAMessage("NetBeansProfiler.handleShutdown.noResults", //NOI18N
                                                                                    TERMINATE_VM_ON_EXIT_MSG,
@@ -1785,12 +1787,10 @@ public final class NetBeansProfiler extends Profiler {
     }
 
     private void cleanupBeforeProfiling(ProfilerEngineSettings sharedSettings) {
-        final Profiler profiler = Profiler.getDefault();
-
         SwingUtilities.invokeLater(new Runnable() {
             public void run() {
-                profiler.getThreadsManager().reset();
-                profiler.getVMTelemetryManager().reset();
+                NetBeansProfiler.this.getThreadsManager().reset();
+                NetBeansProfiler.this.getVMTelemetryManager().reset();
                 if (LiveResultsWindow.hasDefault())
                     LiveResultsWindow.getDefault().handleCleanupBeforeProfiling();
             }
