@@ -1,7 +1,10 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright 2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
+ *
+ * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
+ * Other names may be trademarks of their respective owners.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common
@@ -13,9 +16,9 @@
  * specific language governing permissions and limitations under the
  * License.  When distributing the software, include this License Header
  * Notice in each file and include the License file at
- * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
+ * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the GPL Version 2 section of the License file that
+ * by Oracle in the GPL Version 2 section of the License file that
  * accompanied this code. If applicable, add the following below the
  * License Header, with the fields enclosed by brackets [] replaced by
  * your own identifying information:
@@ -213,23 +216,7 @@ public class OQLController extends AbstractTopLevelController
                         try {
                             analysisRunning.compareAndSet(false, true);
                             queryController.queryStarted(progressModel);
-                            progressUpdater.submit(new Runnable() {
-
-                                public void run() {
-                                    while(analysisRunning.get()) {
-                                        int val = progressModel.getValue() + 10;
-                                        if (val > progressModel.getMaximum()) {
-                                            val = progressModel.getMinimum();
-                                        }
-                                        progressModel.setValue(val);
-                                        try {
-                                            Thread.sleep(200);
-                                        } catch (InterruptedException e) {
-                                            Thread.currentThread().interrupt();
-                                        }
-                                    }
-                                }
-                            });
+                            progressUpdater.submit(new ProgressUpdater(progressModel));
                             engine.executeQuery(oqlQuery, new ObjectVisitor() {
 
                                 public boolean visit(Object o) {
@@ -415,8 +402,12 @@ public class OQLController extends AbstractTopLevelController
             this.oqlController = oqlController;
         }
 
-        public void setResult(String result) {
-            ((OQLControllerUI.ResultsUI)getPanel()).setResult(result);
+        public void setResult(final String result) {
+            SwingUtilities.invokeLater(new Runnable() {
+                public void run() {
+                    ((OQLControllerUI.ResultsUI)getPanel()).setResult(result);
+                }
+            });
         }
 
         public void showURL(URL url) {
@@ -503,10 +494,7 @@ public class OQLController extends AbstractTopLevelController
             OQLSupport.saveModel(model);
         }
 
-
         
-        
-
         protected AbstractButton createControllerPresenter() {
             return ((OQLControllerUI.SavedUI)getPanel()).getPresenter();
         }
@@ -518,4 +506,35 @@ public class OQLController extends AbstractTopLevelController
 
     }
 
+    private class ProgressUpdater implements Runnable {
+
+        private final BoundedRangeModel progressModel;
+
+        ProgressUpdater(BoundedRangeModel model) {
+            progressModel = model;
+        }
+
+        public void run() {
+            while (analysisRunning.get()) {
+                final int newVal;
+                int val = progressModel.getValue() + 10;
+                
+                if (val > progressModel.getMaximum()) {
+                    val = progressModel.getMinimum();
+                }
+                newVal = val;
+                SwingUtilities.invokeLater(new Runnable() {
+                    public void run() {
+                        progressModel.setValue(newVal);
+                    }
+                });
+                try {
+                    Thread.sleep(200);
+                } catch (InterruptedException e) {
+                    Thread.currentThread().interrupt();
+                }
+            }
+        }
+    }
+
 }
